<?php

namespace Server\Controllers;

use Server\Library\Controllers\SolidController;

use Server\Database\Models\User;
use Server\Database\Models\Pause;
use Server\Database\Models\Contract;
use Server\Controllers\Filters\ContractsFilter;
use Server\Controllers\Validators\ContractsValidator;

class ContractsController extends SolidController
{

    public function __construct()
    {
        $this->model = new Contract;
        $this->filter = new ContractsFilter;
        $this->validator = new ContractsValidator(new Contract);
    }

    protected function filter($body, $keysWhitelist)
    {
        return array_filter($body, function ($item, $key) use ($keysWhitelist) {
            return in_array($key, $keysWhitelist);
        }, ARRAY_FILTER_USE_BOTH);
    }

    public function stop($request, $response)
    {
        $body = $request->getParsedBody();
        $id = $body['id'] ?? '';
        $user_id = $body['user_id'] ?? '';

        $rules = [
            'id' => [$id, 'required'],
            'user_id' => [$user_id, 'required'],
        ];

        $this->validator->validate($rules);
        $errors = $this->validator->errors()->all();

        if ($errors) {
            $this->data['errors'] = $errors;
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $row = Contract::where('id', $id)->where('user_id', $user_id)->first();

        if (!$row) {
            $this->data['errors'] = ['not found'];
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $timestamp = time();
        $row->update(['end_timestamp' => $timestamp, 'status' => 2]);

        $user = User::where('id', $user_id)->first();
        $user = User::relationships($user);

        $this->data['data'] = $user;
        $response->getBody()->write(json_encode($this->data));
        return $response->withHeader('Content-Type', 'application/json');
    }

    public function pause($request, $response)
    {
        // create pause using contract id
        $body = $request->getParsedBody();
        $user_id = $body['user_id'] ?? '';
        $contract_id = $body['contract_id'] ?? '';
        $speed_per_day = $body['speed_per_day'] ?? '';

        $rules =  [
            'user id' => [$user_id, 'required'],
            'contract id' => [$contract_id, 'required'],
            'speed per id' => [$speed_per_day, 'required']
        ];

        $this->validator->validate($rules);
        $errors = $this->validator->errors()->all();

        if ($errors) {
            $this->data['errors'] = $errors;
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $body['start_timestamp'] = time();

        $body = $this->filter($body, ['user_id', 'contract_id', 'start_timestamp', 'speed_per_day']);

        Pause::create($body);

        $row = Contract::where('id', $contract_id)->first();

        if (!$row) {
            $this->data['errors'] = ['not found'];
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $row->update(['status' => 3]);

        $user = User::where('id', $user_id)->first();
        $user = User::relationships($user);

        $this->data['data'] = $user;

        $response->getBody()->write(json_encode($this->data));
        return $response->withHeader('Content-Type', 'application/json');
    }

    public function play($request, $response)
    {
        // check if contract is paused using pause id
        $body = $request->getParsedBody();
        $user_id = $body['user_id'] ?? '';
        $contract_id = $body['contract_id'] ?? '';

        $rules = [
            'user id' => [$user_id, 'required'],
            'contract id' => [$contract_id, 'required'],
        ];

        $this->validator->validate($rules);
        $errors = $this->validator->errors()->all();

        if ($errors) {
            $this->data['errors'] = $errors;
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $row = Contract::where('id', $contract_id)->first();

        if (!$row) {
            $this->data['errors'] = ['not found'];
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $row->update(['status' => 1]);

        $row = Pause::where('contract_id', $contract_id)->where('end_timestamp', null)->first();

        if (!$row) {
            $this->data['errors'] = ['not found'];
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $timestamp = time();
        $row->update(['end_timestamp' => $timestamp]);

        $user = User::where('id', $user_id)->first();
        $user = User::relationships($user);

        $this->data['data'] = $user;
        $response->getBody()->write(json_encode($this->data));
        return $response->withHeader('Content-Type', 'application/json');
    }

}
