<?php

namespace Server\Controllers;

use Server\Database\Models\User;
use Server\Database\Models\Deposit;
use Server\Controllers\Filters\DepositsFilter;
use Server\Library\Controllers\SolidController;
use Server\Controllers\Validators\DepositsValidator;

class DepositsController extends SolidController
{

    public function __construct()
    {
        $this->model = new Deposit;
        $this->filter = new DepositsFilter;
        $this->validator = new DepositsValidator(new Deposit);
    }

    public function uploadProof($request, $response) {

        $id = $request->getAttribute('id');
        $user = $request->getAttribute('user');

        $proof_name = $_FILES['proof']['name'] ?? '';
        $proof_size = $_FILES['proof']['size'] ?? 0;

        $rules = [
            'id' => [$id,'required'],
            'proof' => [$proof_name, 'required|imageformat'],
            'proof' => [$proof_size, 'imagesize'],
        ];

        $this->validator->validate($rules);
        $errors = $this->validator->errors()->all();

        if ($errors) {
            $this->data['errors'] = $errors;
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $transfer = $this->model->where('id',$id)->first();
        if (!$transfer) {
            $this->data['errors'] = ['Transfer Not Found'];
            $response->getBody()->write(json_encode($this->data));
            return $response->withHeader('Content-Type', 'application/json');
        }

        $proof = $this->uploadImage($_FILES['proof']);
        $transfer->update(['proof' => $proof]);

        $user = User::where('id', $user->id)->first();
        $user = User::relationships($user);

        $this->data['data'] = $user;
        $this->data['message'] = "Upload Successful";

        $response->getBody()->write(json_encode($this->data));
        return $response->withHeader('Content-Type', 'application/json');

    }
}