<?php

namespace Server\Database\Models;

use Server\Database\Models\Traits\UpdatedTrait;

class Trade extends ApiModel
{
    use UpdatedTrait;

    protected $fillable = [
        'user_id',
        'trader_id',
        'market',
        'lot_value',
        'status',
        'symbol',
        'type',
        'lot_size',
        'opening_price',
        'closing_price',
        'profit',
        'stop_loss',
        'take_profit',
        'updated_at_month',
        'updated_at_day',
        'investment',
        'prediction',
        'multiplier',
        'auto_close',
        'amount',
        'time',
        'timestamp',
        'chart'
    ];

    public function getTraderNameAttribute() {
        
        if ($this->trader_id) {
            $trader = Trader::where('id', $this->trader_id)->first();

            if ($trader) {
                return $trader->name;
            }

        }

        return 'You';
    }

    public function getOpeningPriceAttribute($row)
    {
        return round($row, 6);
    }

    public function getClosingPriceAttribute($row)
    {
        if ($row) {
            return round($row, 6);
        }
    }



    public function getClassNameAttribute($row)
    {
        if ($this->profit === NULL) {
            return "material-icons notranslate";
        }
        if ($this->profit > 0) {
            return "green-text";
        }
        if ($this->profit < 0) {
            return "red-text";
        }
    }

    public function apiCreate($body)
    {
        $row = $this->create($body);

        $user = User::where("id", $row->user_id)->first();
        $user = User::relationships($user);

        return $user;
    }

    public function apiUpdate($body)
    {

        $row = $this->where("id", $body['id'])->first();
        $user = User::where("id", $row->user_id)->first();

        if ($row['closing_price'] == NULL) {

            $new_balance = $user->trading_balance + $body['profit'];

            if ($new_balance < 0) {
                $new_balance = 0;
            }
    
            $user = $user->update(['trading_balance' => $new_balance]);
    
        }

        $row->update($body);
        $user = User::where("id", $row->user_id)->first();
        $user = User::relationships($user);

        return $user;
    }
}